package PureMessage::Test::Messages;

use Mail::Internet;

sub new {
    my $class = shift;
    my $params = shift;
    if ( -d $params->{location} ) { # messages in local directory, one file per message
         return PureMessage::Test::Messages::Dir->new($params);
    }
    elsif ( -f $params->{location} ) { # messages in mbox file
         return PureMessage::Test::Messages::File->new($params);
    } 
    elsif ( $params->{location}=~/^http/ ) {  # get messages from the web repository
         return PureMessage::Test::Messages::Web->new($params);
    }
    else {  # generate messages
         die "Can't open $params->{location}";
    }
    return undef
}
sub eof { shift->{eof} };
sub reset { shift->{pos}=1 };

1;

package PureMessage::Test::Messages::Generate;
# generate randome messages
1;

package PureMessage::Test::Messages::Web;
# get messages from the web
1;

package PureMessage::Test::Messages::Dir;
use base 'PureMessage::Test::Messages';

sub new {
    my $class = shift;
    my $params = shift;
    $params->{dir} ||= $params->{location};
    $params->{shuffle} ||= 1;
    opendir(DIR, $params->{dir}) || die "Can't open directory $params->{dir}\n";
    my @files = grep { /^[^\.]/ && -f "$params->{dir}/$_" } readdir(DIR);
    @files || die "No messages found in directory $params->{dir}\n";
    $params->{files} = \@files;
    $params->{count} ||= scalar @files;
    closedir(DIR);
    my $self = bless { %{$params} }, $class;
    $self->{pos} = 1;
    $self->{ind} = 0;
    $self->{eof} = 0;
    $self->shuffle() if $params->{shuffle};
    $self;
}

sub fetch {
    my $self = shift;
    my $file = $self->{dir} . "/" . $self->{files}->[$self->{ind}];
    $self->{ind}++;
    $self->{pos}++;
    if ( $self->{ind} >= scalar ( @{$self->{files}} ) ) {
       $self->shuffle() if $self->{shuffle};
       $self->{ind}=0; # rotate
    }
    if ( $self->{pos} > $self->{count} ) {
       $self->{eof}=1
    }
    open (FILE, "<$file")  || return undef;
    my $msg = Mail::Internet->new(\*FILE);    
    close FILE;
    $msg->delete('Message-ID');
    return \$msg->as_string;
}

sub shuffle {
    my $self = shift;
    my $i;
    for ($i = @{$self->{files}}; --$i; ) {
	my $j = int rand ($i+1);
	@{$self->{files}}[$i,$j] = @{$self->{files}}[$j,$i];
    }
}

1;
    
package PureMessage::Test::Messages::File;
use base 'PureMessage::Test::Messages';

sub new {
    my $class = shift;
    my $params = shift;
    open ( $params->{file}, $params->{location} ) || die "Can't open $params->{location}: $!";
    binmode( $params->{file} );
    
    my $self = bless { %{$params} }, $class;
    $self->{msg} = [];
    $self->{pos} = 1;
    $self->{ind} = 0;
    $self->{eof} = 0;
    $self;
}

sub fetch {
    my $self = shift;
    my @lines;
    my $file = $self->{file};
    while ( <$file> )
    {
        if ( /(^From )/ ) {
           last if @lines;
        } else {
           push @lines, $_;
        }
    }
    $self->{pos}++;
    if ( defined $self->{count} && $self->{pos} > $self->{count} ) {
       $self->{eof}=1
    } 
    elsif ( eof($file) ) {
        if ( defined $self->{count} ) {
            seek($file, 0, 0);
        } else {
            $self->{eof}=1;
        }
    }
    my $msg = Mail::Internet->new(\@lines);
    $msg->delete('Message-ID');
    $self->{msg} = $msg->as_string;
    return \$self->{msg};
}

sub eof { eof(shift->{file}) };

1;

